package de.mlessmann.certassist;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

import de.mlessmann.certassist.openssl.*;
import de.mlessmann.certassist.openssl.CertificateRequest.RequestType;
import java.nio.file.Path;
import java.util.Objects;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class TestOpenSSLCertificateCreator {

    private CertificatePasswordProvider passwordProvider;

    @BeforeEach
    void setUp() {
        passwordProvider = mock(CertificatePasswordProvider.class);
        when(passwordProvider.generateNewPassword()).thenReturn("ABC-123");
    }

    @Test
    void testCertificateCreation() throws Exception {
        CertificateProvider certificateProvider = mock(CertificateProvider.class);
        ExecutableResolver executableResolver = new ExecutableResolver();
        var certificateCreator = new OpenSSLCertificateCreator(
            executableResolver,
            passwordProvider,
            certificateProvider
        );

        CertificateRequest certRequest = CertificateRequest
            .builder()
            .commonName("test.home")
            .type(RequestType.STANDALONE_CERTIFICATE)
            .subject(CertificateSubject.builder().country("DE").state("SH").locality("HH").organization("Crazy-Cats"))
            .extension(CertificateRequestExtension.builder().alternativeNames("test2.home", "test3.home"))
            .build();

        try (var cert = certificateCreator.createCertificate(certRequest)) {
            assertThat(certificateCreator.verifyCertificate(cert.certificatePath(), cert.certificatePath()))
                .isEqualTo(true);
            System.out.println("Certificate created: " + cert);

            CertificateRequest childRequest = CertificateRequest
                .builder()
                .commonName("test.local")
                .type(RequestType.NORMAL_CERTIFICATE)
                .trustingAuthority(cert.fingerprint())
                .subject(
                    CertificateSubject.builder().country("DE").state("SH").locality("HH").organization("Crazy-Cats")
                )
                .build();

            var spiedCert = spy(cert);
            doNothing().when(spiedCert).close();
            when(certificateProvider.requestCertificateUsage(cert.fingerprint())).thenReturn(spiedCert);
            try (var childCert = certificateCreator.createCertificate(childRequest)) {
                System.out.println("Child certificate created: " + childCert);
                Path fullchain = childCert.fullchainPath();
                assertThat(
                    certificateCreator.verifyCertificate(cert.certificatePath(), Objects.requireNonNull(fullchain))
                )
                    .isEqualTo(true);
            }
        }
    }
}
