package de.mlessmann.certassist.service;

import de.mlessmann.certassist.openssl.CertificatePasswordProvider;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.stereotype.Service;

@Service
@ConditionalOnMissingBean(CertificatePasswordProvider.class)
public class InMemoryCertificatePasswordProvider implements CertificatePasswordProvider {

    private final Map<String, String> passwords = new ConcurrentHashMap<>();
    private static final int PASSPHRASE_LENGTH = 16;

    @Override
    public String generateNewPassword() {
        return PassphraseService.generateNewPassword(PASSPHRASE_LENGTH);
    }

    @Override
    public String getPasswordFor(String certificateFingerprint) {
        return Optional.ofNullable(passwords.get(certificateFingerprint)).orElseThrow();
    }

    @Override
    public void setPasswordFor(String certificateFingerprint, String password) {
        Objects.requireNonNull(certificateFingerprint);
        Objects.requireNonNull(password);
        passwords.put(certificateFingerprint, password);
    }
}
